@@ -88,12 +88,12 @@ idmef_class_child_id_t idmef_class_find_child(idmef_class_id_t class, const char
                 return ret;
 
         list = object_data[class].children_list;
-	
-	if ( list ) {
-		for ( i = 0; list[i].name; i++ )
-			if ( strcasecmp(list[i].name, name) == 0)
-				return i;
-	}
+
+        if ( list ) {
+                for ( i = 0; list[i].name; i++ )
+                        if ( strcasecmp(list[i].name, name) == 0)
+                                return i;
+        }
 
         return prelude_error_verbose(PRELUDE_ERROR_IDMEF_CLASS_UNKNOWN_CHILD, "Unknown IDMEF child '%s'", name);
 }
@@ -230,6 +230,18 @@ int idmef_class_new_child(void *ptr, idmef_class_id_t class, idmef_class_child_i
 
 
 
+int idmef_class_destroy_child(void *ptr, idmef_class_id_t class, idmef_class_child_id_t child, int n)
+{
+        int ret;
+
+        ret = is_child_valid(class, child);
+        if ( ret < 0 )
+                return ret;
+
+        return object_data[class].destroy_child(ptr, child, n);
+}
+
+
 int idmef_class_copy(idmef_class_id_t class, const void *src, void *dst)
 {
         int ret;
