@@ -84,7 +84,7 @@
      TOK_RELATION_EQUAL_NOCASE = 273,
      TOK_RELATION_NOT_EQUAL = 274,
      TOK_RELATION_NOT_EQUAL_NOCASE = 275,
-     TOK_RELATION_IS_NULL = 276,
+     TOK_NOT = 276,
      TOK_OPERATOR_AND = 277,
      TOK_OPERATOR_OR = 278,
      TOK_ERROR = 279
@@ -109,7 +109,7 @@
 #define TOK_RELATION_EQUAL_NOCASE 273
 #define TOK_RELATION_NOT_EQUAL 274
 #define TOK_RELATION_NOT_EQUAL_NOCASE 275
-#define TOK_RELATION_IS_NULL 276
+#define TOK_NOT 276
 #define TOK_OPERATOR_AND 277
 #define TOK_OPERATOR_OR 278
 #define TOK_ERROR 279
@@ -138,9 +138,12 @@
 #include "idmef-criteria.h"
 #include "common.h"
 
-
+static int path_count = 0;
 static int real_ret = 0;
+static idmef_path_t *cur_path;
 static idmef_criteria_t *processed_criteria;
+static idmef_criterion_operator_t cur_operator;
+
 pthread_mutex_t _criteria_parse_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 
@@ -158,6 +161,36 @@ void _idmef_criteria_string_init_lexer(void);
 #define YYERROR_VERBOSE
 
 
+static int create_criteria(idmef_criteria_t **criteria, idmef_path_t *path,
+                           idmef_criterion_value_t *value, idmef_criterion_operator_t operator)
+{
+        idmef_criterion_t *criterion;
+
+        real_ret = idmef_criteria_new(criteria);
+        if ( real_ret < 0 )
+                goto err;
+
+        if ( path_count++ > 0 )
+                idmef_path_ref(path);
+
+        real_ret = idmef_criterion_new(&criterion, path, value, operator);
+        if ( real_ret < 0 ) {
+                idmef_criteria_destroy(*criteria);
+                goto err;
+        }
+
+        idmef_criteria_set_criterion(*criteria, criterion);
+        return 0;
+
+err:
+        idmef_path_destroy(path);
+        cur_path = NULL;
+
+        return real_ret;
+}
+
+
+
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -179,16 +212,16 @@ void _idmef_criteria_string_init_lexer(void);
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 65 "idmef-criteria-string.yac.y"
+#line 98 "idmef-criteria-string.yac.y"
 {
         char *str;
         int operator;
-        idmef_criterion_t *criterion;
+        idmef_path_t *path;
         idmef_criteria_t *criteria;
         idmef_criterion_operator_t relation;
 }
 /* Line 187 of yacc.c.  */
-#line 192 "idmef-criteria-string.yac.c"
+#line 225 "idmef-criteria-string.yac.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -201,7 +234,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 205 "idmef-criteria-string.yac.c"
+#line 238 "idmef-criteria-string.yac.c"
 
 #ifdef short
 # undef short
@@ -414,18 +447,18 @@ union yyalloc
 #endif
 
 /* YYFINAL -- State number of the termination state.  */
-#define YYFINAL  29
+#define YYFINAL  12
 /* YYLAST -- Last index in YYTABLE.  */
-#define YYLAST   41
+#define YYLAST   54
 
 /* YYNTOKENS -- Number of terminals.  */
 #define YYNTOKENS  27
 /* YYNNTS -- Number of nonterminals.  */
-#define YYNNTS  7
+#define YYNNTS  10
 /* YYNRULES -- Number of rules.  */
-#define YYNRULES  29
+#define YYNRULES  36
 /* YYNRULES -- Number of states.  */
-#define YYNSTATES  36
+#define YYNSTATES  50
 
 /* YYTRANSLATE(YYLEX) -- Bison symbol number corresponding to YYLEX.  */
 #define YYUNDEFTOK  2
@@ -472,29 +505,34 @@ static const yytype_uint8 yytranslate[] =
    YYRHS.  */
 static const yytype_uint8 yyprhs[] =
 {
-       0,     0,     3,     5,     7,    11,    13,    17,    21,    23,
-      26,    28,    30,    32,    34,    36,    38,    40,    42,    44,
-      46,    48,    50,    52,    54,    56,    58,    60,    62,    64
+       0,     0,     3,     5,     7,    11,    13,    17,    22,    28,
+      32,    34,    37,    39,    41,    45,    49,    51,    53,    55,
+      57,    59,    61,    63,    65,    67,    69,    71,    73,    75,
+      77,    79,    81,    83,    85,    87,    89
 };
 
 /* YYRHS -- A `-1'-separated list of the rules' RHS.  */
 static const yytype_int8 yyrhs[] =
 {
-      28,     0,    -1,    29,    -1,    30,    -1,    29,    33,    30,
-      -1,    31,    -1,    25,    29,    26,    -1,     4,    32,     3,
-      -1,     4,    -1,    21,     4,    -1,     5,    -1,     6,    -1,
-       7,    -1,     8,    -1,     9,    -1,    10,    -1,    11,    -1,
-      12,    -1,    13,    -1,    14,    -1,    15,    -1,    16,    -1,
-      17,    -1,    18,    -1,    19,    -1,    20,    -1,    21,    -1,
-      24,    -1,    22,    -1,    23,    -1
+      28,     0,    -1,    29,    -1,    30,    -1,    29,    36,    30,
+      -1,    31,    -1,    25,    29,    26,    -1,    21,    25,    29,
+      26,    -1,    32,    35,    25,    34,    26,    -1,    32,    35,
+      33,    -1,    32,    -1,    21,    32,    -1,     4,    -1,     3,
+      -1,    34,    36,    34,    -1,    25,    34,    26,    -1,    33,
+      -1,     5,    -1,     6,    -1,     7,    -1,     8,    -1,     9,
+      -1,    10,    -1,    11,    -1,    12,    -1,    13,    -1,    14,
+      -1,    15,    -1,    16,    -1,    17,    -1,    18,    -1,    19,
+      -1,    20,    -1,    21,    -1,    24,    -1,    22,    -1,    23,
+      -1
 };
 
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   117,   117,   122,   125,   136,   147,   152,   187,   210,
-     235,   236,   237,   238,   239,   240,   241,   242,   243,   244,
-     245,   246,   247,   248,   249,   250,   251,   252,   256,   257
+       0,   153,   153,   160,   164,   175,   179,   183,   191,   195,
+     199,   209,   222,   236,   257,   266,   270,   277,   278,   279,
+     280,   281,   282,   283,   284,   285,   286,   287,   288,   289,
+     290,   291,   292,   293,   294,   298,   299
 };
 #endif
 
@@ -503,18 +541,12 @@ static const yytype_uint16 yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "TOK_IDMEF_VALUE", "TOK_IDMEF_PATH",
-  "TOK_RELATION_SUBSTRING", "TOK_RELATION_SUBSTRING_NOCASE",
-  "TOK_RELATION_NOT_SUBSTRING", "TOK_RELATION_NOT_SUBSTRING_NOCASE",
-  "TOK_RELATION_REGEXP", "TOK_RELATION_REGEXP_NOCASE",
-  "TOK_RELATION_NOT_REGEXP", "TOK_RELATION_NOT_REGEXP_NOCASE",
-  "TOK_RELATION_GREATER", "TOK_RELATION_GREATER_OR_EQUAL",
-  "TOK_RELATION_LESS", "TOK_RELATION_LESS_OR_EQUAL", "TOK_RELATION_EQUAL",
-  "TOK_RELATION_EQUAL_NOCASE", "TOK_RELATION_NOT_EQUAL",
-  "TOK_RELATION_NOT_EQUAL_NOCASE", "TOK_RELATION_IS_NULL",
-  "TOK_OPERATOR_AND", "TOK_OPERATOR_OR", "TOK_ERROR", "'('", "')'",
-  "$accept", "input", "criteria", "criteria_base", "criterion", "relation",
-  "operator", 0
+  "$end", "error", "$undefined", "\"<IDMEF-Value>\"", "\"<IDMEF-Path>\"",
+  "\"<>\"", "\"<>*\"", "\"!<>\"", "\"!<>*\"", "\"~\"", "\"~*\"", "\"!~\"",
+  "\"!~*\"", "\">\"", "\">=\"", "\"<\"", "\"<=\"", "\"=\"", "\"=*\"",
+  "\"!=\"", "\"!=*\"", "\"!\"", "\"&&\"", "\"||\"", "TOK_ERROR", "'('",
+  "')'", "$accept", "input", "criteria", "criteria_base", "criterion",
+  "path", "value", "multiple_value", "relation", "operator", 0
 };
 #endif
 
@@ -532,17 +564,19 @@ static const yytype_uint16 yytoknum[] =
 /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
 static const yytype_uint8 yyr1[] =
 {
-       0,    27,    28,    29,    29,    30,    30,    31,    31,    31,
-      32,    32,    32,    32,    32,    32,    32,    32,    32,    32,
-      32,    32,    32,    32,    32,    32,    32,    32,    33,    33
+       0,    27,    28,    29,    29,    30,    30,    30,    31,    31,
+      31,    31,    32,    33,    34,    34,    34,    35,    35,    35,
+      35,    35,    35,    35,    35,    35,    35,    35,    35,    35,
+      35,    35,    35,    35,    35,    36,    36
 };
 
 /* YYR2[YYN] -- Number of symbols composing right hand side of rule YYN.  */
 static const yytype_uint8 yyr2[] =
 {
-       0,     2,     1,     1,     3,     1,     3,     3,     1,     2,
+       0,     2,     1,     1,     3,     1,     3,     4,     5,     3,
+       1,     2,     1,     1,     3,     3,     1,     1,     1,     1,
        1,     1,     1,     1,     1,     1,     1,     1,     1,     1,
-       1,     1,     1,     1,     1,     1,     1,     1,     1,     1
+       1,     1,     1,     1,     1,     1,     1
 };
 
 /* YYDEFACT[STATE-NAME] -- Default rule to reduce with in state
@@ -550,33 +584,35 @@ static const yytype_uint8 yyr2[] =
    means the default is an error.  */
 static const yytype_uint8 yydefact[] =
 {
-       0,     8,     0,     0,     0,     2,     3,     5,    10,    11,
-      12,    13,    14,    15,    16,    17,    18,    19,    20,    21,
-      22,    23,    24,    25,    26,    27,     0,     9,     0,     1,
-      28,    29,     0,     7,     6,     4
+       0,    12,     0,     0,     0,     2,     3,     5,    10,     0,
+      11,     0,     1,    35,    36,     0,    17,    18,    19,    20,
+      21,    22,    23,    24,    25,    26,    27,    28,    29,    30,
+      31,    32,    33,    34,     0,     0,     6,     4,    13,     0,
+       9,     7,     0,    16,     0,     0,     8,     0,    15,    14
 };
 
 /* YYDEFGOTO[NTERM-NUM].  */
 static const yytype_int8 yydefgoto[] =
 {
-      -1,     4,     5,     6,     7,    26,    32
+      -1,     4,     5,     6,     7,     8,    43,    44,    34,    15
 };
 
 /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
    STATE-NUM.  */
-#define YYPACT_NINF -23
+#define YYPACT_NINF -38
 static const yytype_int8 yypact[] =
 {
-      -4,    17,     2,    -4,     7,   -19,   -23,   -23,   -23,   -23,
-     -23,   -23,   -23,   -23,   -23,   -23,   -23,   -23,   -23,   -23,
-     -23,   -23,   -23,   -23,   -23,   -23,     5,   -23,   -21,   -23,
-     -23,   -23,    -4,   -23,   -23,   -23
+      -1,   -38,     0,    -1,     6,    31,   -38,   -38,    21,    -1,
+     -38,    -7,   -38,   -38,   -38,    -1,   -38,   -38,   -38,   -38,
+     -38,   -38,   -38,   -38,   -38,   -38,   -38,   -38,   -38,   -38,
+     -38,   -38,   -38,   -38,    -3,    -5,   -38,   -38,   -38,    -2,
+     -38,   -38,    -2,   -38,    24,    26,   -38,    -2,   -38,    31
 };
 
 /* YYPGOTO[NTERM-NUM].  */
 static const yytype_int8 yypgoto[] =
 {
-     -23,   -23,     6,   -22,   -23,   -23,   -23
+     -38,   -38,     2,   -13,   -38,     8,   -21,   -33,   -38,   -37
 };
 
 /* YYTABLE[YYPACT[STATE-NUM]].  What to do in state STATE-NUM.  If
@@ -586,30 +622,33 @@ static const yytype_int8 yypgoto[] =
 #define YYTABLE_NINF -1
 static const yytype_uint8 yytable[] =
 {
-       1,    30,    31,    30,    31,    34,    27,    29,    33,    28,
-      35,     0,     0,     0,     0,     0,     0,     2,     0,     0,
-       0,     3,     8,     9,    10,    11,    12,    13,    14,    15,
-      16,    17,    18,    19,    20,    21,    22,    23,    24,     0,
-       0,    25
+      38,    38,    37,     1,     1,    11,    12,    47,    47,    45,
+      10,    35,    47,    40,    49,    13,    14,    13,    14,    36,
+       2,    41,    39,    42,     3,     9,    16,    17,    18,    19,
+      20,    21,    22,    23,    24,    25,    26,    27,    28,    29,
+      30,    31,    32,     0,     0,    33,    13,    14,    13,    14,
+      46,     0,    48,    13,    14
 };
 
 static const yytype_int8 yycheck[] =
 {
-       4,    22,    23,    22,    23,    26,     4,     0,     3,     3,
-      32,    -1,    -1,    -1,    -1,    -1,    -1,    21,    -1,    -1,
-      -1,    25,     5,     6,     7,     8,     9,    10,    11,    12,
-      13,    14,    15,    16,    17,    18,    19,    20,    21,    -1,
-      -1,    24
+       3,     3,    15,     4,     4,     3,     0,    44,    45,    42,
+       2,     9,    49,    34,    47,    22,    23,    22,    23,    26,
+      21,    26,    25,    25,    25,    25,     5,     6,     7,     8,
+       9,    10,    11,    12,    13,    14,    15,    16,    17,    18,
+      19,    20,    21,    -1,    -1,    24,    22,    23,    22,    23,
+      26,    -1,    26,    22,    23
 };
 
 /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
    symbol of state STATE-NUM.  */
 static const yytype_uint8 yystos[] =
 {
-       0,     4,    21,    25,    28,    29,    30,    31,     5,     6,
-       7,     8,     9,    10,    11,    12,    13,    14,    15,    16,
-      17,    18,    19,    20,    21,    24,    32,     4,    29,     0,
-      22,    23,    33,     3,    26,    30
+       0,     4,    21,    25,    28,    29,    30,    31,    32,    25,
+      32,    29,     0,    22,    23,    36,     5,     6,     7,     8,
+       9,    10,    11,    12,    13,    14,    15,    16,    17,    18,
+      19,    20,    21,    24,    35,    29,    26,    30,     3,    25,
+      33,    26,    25,    33,    34,    34,    26,    36,    26,    34
 };
 
 #define yyerrok		(yyerrstatus = 0)
@@ -1117,20 +1156,20 @@ yydestruct (yymsg, yytype, yyvaluep)
 
   switch (yytype)
     {
-      case 3: /* "TOK_IDMEF_VALUE" */
-#line 78 "idmef-criteria-string.yac.y"
+      case 3: /* "\"<IDMEF-Value>\"" */
+#line 111 "idmef-criteria-string.yac.y"
 	{ free ((yyvaluep->str)); };
-#line 1124 "idmef-criteria-string.yac.c"
+#line 1163 "idmef-criteria-string.yac.c"
 	break;
-      case 4: /* "TOK_IDMEF_PATH" */
-#line 78 "idmef-criteria-string.yac.y"
+      case 4: /* "\"<IDMEF-Path>\"" */
+#line 111 "idmef-criteria-string.yac.y"
 	{ free ((yyvaluep->str)); };
-#line 1129 "idmef-criteria-string.yac.c"
+#line 1168 "idmef-criteria-string.yac.c"
 	break;
       case 29: /* "criteria" */
-#line 79 "idmef-criteria-string.yac.y"
+#line 112 "idmef-criteria-string.yac.y"
 	{ idmef_criteria_destroy((yyvaluep->criteria)); };
-#line 1134 "idmef-criteria-string.yac.c"
+#line 1173 "idmef-criteria-string.yac.c"
 	break;
 
       default:
@@ -1439,250 +1478,257 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 117 "idmef-criteria-string.yac.y"
+#line 153 "idmef-criteria-string.yac.y"
     {
-                                                        processed_criteria = (yyvsp[(1) - (1)].criteria);
-                                                }
+                processed_criteria = (yyvsp[(1) - (1)].criteria);
+        }
     break;
 
   case 3:
-#line 122 "idmef-criteria-string.yac.y"
+#line 160 "idmef-criteria-string.yac.y"
     {
-                                                        (yyval.criteria) = (yyvsp[(1) - (1)].criteria);
-                                                }
+                (yyval.criteria) = (yyvsp[(1) - (1)].criteria);
+        }
     break;
 
   case 4:
-#line 125 "idmef-criteria-string.yac.y"
+#line 164 "idmef-criteria-string.yac.y"
     {
-                                                        if ( (yyvsp[(2) - (3)].operator) == operator_or ) {
-                                                                idmef_criteria_or_criteria((yyvsp[(1) - (3)].criteria), (yyvsp[(3) - (3)].criteria));
-                                                        } else {
-                                                                idmef_criteria_and_criteria((yyvsp[(1) - (3)].criteria), (yyvsp[(3) - (3)].criteria));
-                                                        }
+                if ( (yyvsp[(2) - (3)].operator) == operator_or )
+                        idmef_criteria_or_criteria((yyvsp[(1) - (3)].criteria), (yyvsp[(3) - (3)].criteria));
+                else
+                        idmef_criteria_and_criteria((yyvsp[(1) - (3)].criteria), (yyvsp[(3) - (3)].criteria));
 
-                                                        (yyval.criteria) = (yyvsp[(1) - (3)].criteria);
-                                                }
+                (yyval.criteria) = (yyvsp[(1) - (3)].criteria);
+        }
     break;
 
   case 5:
-#line 136 "idmef-criteria-string.yac.y"
+#line 175 "idmef-criteria-string.yac.y"
     {
-                                                        idmef_criteria_t *criteria;
-
-                                                        real_ret = idmef_criteria_new(&criteria);
-                                                        if ( real_ret < 0 )
-                                                                YYABORT;
-
-                                                        idmef_criteria_set_criterion(criteria, (yyvsp[(1) - (1)].criterion));
-                                                        (yyval.criteria) = criteria;
-
-                                                }
+                (yyval.criteria) = (yyvsp[(1) - (1)].criteria);
+        }
     break;
 
   case 6:
-#line 147 "idmef-criteria-string.yac.y"
+#line 179 "idmef-criteria-string.yac.y"
     {
-                                                        (yyval.criteria) = (yyvsp[(2) - (3)].criteria);
-                                                }
+                (yyval.criteria) = (yyvsp[(2) - (3)].criteria);
+        }
     break;
 
   case 7:
-#line 152 "idmef-criteria-string.yac.y"
+#line 183 "idmef-criteria-string.yac.y"
     {
-                                                        idmef_path_t *path = NULL;
-                                                        idmef_criterion_value_t *value = NULL;
-                                                        idmef_criterion_operator_t operator = (yyvsp[(2) - (3)].relation);
-                                                        idmef_criterion_t *criterion;
-
-                                                        real_ret = idmef_path_new_fast(&path, (yyvsp[(1) - (3)].str));
-                                                        if ( real_ret < 0 ) {
-                                                                free((yyvsp[(1) - (3)].str));
-                                                                free((yyvsp[(3) - (3)].str));
-                                                                YYABORT;
-                                                        }
-
-                                                        real_ret = idmef_criterion_value_new_from_string(&value, path, (yyvsp[(3) - (3)].str), operator);
-                                                        if ( real_ret < 0 ) {
-                                                                free((yyvsp[(1) - (3)].str));
-                                                                free((yyvsp[(3) - (3)].str));
-                                                                idmef_path_destroy(path);
-                                                                YYABORT;
-                                                        }
-
-                                                        real_ret = idmef_criterion_new(&criterion, path, value, operator);
-                                                        if ( real_ret < 0 ) {
-                                                                free((yyvsp[(1) - (3)].str));
-                                                                free((yyvsp[(3) - (3)].str));
-                                                                idmef_path_destroy(path);
-                                                                idmef_criterion_value_destroy(value);
-                                                                YYABORT;
-                                                        }
-
-                                                        free((yyvsp[(1) - (3)].str));
-                                                        free((yyvsp[(3) - (3)].str));
-
-                                                        (yyval.criterion) = criterion;
-                                                }
+                idmef_criteria_set_negation((yyvsp[(3) - (4)].criteria), TRUE);
+                (yyval.criteria) = (yyvsp[(3) - (4)].criteria);
+        }
     break;
 
   case 8:
-#line 187 "idmef-criteria-string.yac.y"
+#line 191 "idmef-criteria-string.yac.y"
     {
-                                                        idmef_path_t *path;
-                                                        idmef_criterion_t *criterion;
-
-                                                        real_ret = idmef_path_new_fast(&path, (yyvsp[(1) - (1)].str));
-                                                        if ( real_ret < 0 ) {
-                                                                free((yyvsp[(1) - (1)].str));
-                                                                YYABORT;
-                                                        }
-
-                                                        real_ret = idmef_criterion_new(&criterion, path, NULL,
-                                                                                       IDMEF_CRITERION_OPERATOR_NOT|
-                                                                                       IDMEF_CRITERION_OPERATOR_NULL);
-                                                        if ( real_ret < 0 ) {
-                                                                free((yyvsp[(1) - (1)].str));
-                                                                idmef_path_destroy(path);
-                                                                YYABORT;
-                                                        }
-
-                                                        free((yyvsp[(1) - (1)].str));
-
-                                                        (yyval.criterion) = criterion;
-                                                }
+                (yyval.criteria) = (yyvsp[(4) - (5)].criteria);
+        }
     break;
 
   case 9:
-#line 210 "idmef-criteria-string.yac.y"
+#line 195 "idmef-criteria-string.yac.y"
     {
-                                                        idmef_path_t *path;
-                                                        idmef_criterion_t *criterion;
-
-                                                        real_ret = idmef_path_new_fast(&path, (yyvsp[(2) - (2)].str));
-                                                        if ( real_ret < 0) {
-                                                                free((yyvsp[(2) - (2)].str));
-                                                                YYABORT;
-                                                        }
-
-                                                        real_ret = idmef_criterion_new(&criterion, path, NULL,
-                                                                                       IDMEF_CRITERION_OPERATOR_NULL);
-                                                        if ( real_ret < 0 ) {
-                                                                free((yyvsp[(2) - (2)].str));
-                                                                idmef_path_destroy(path);
-                                                                YYABORT;
-                                                        }
-
-                                                        free((yyvsp[(2) - (2)].str));
-
-                                                        (yyval.criterion) = criterion;
-                                                }
+                (yyval.criteria) = (yyvsp[(3) - (3)].criteria);
+        }
     break;
 
   case 10:
-#line 235 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR; }
+#line 199 "idmef-criteria-string.yac.y"
+    {
+                idmef_criteria_t *criteria;
+
+                real_ret = create_criteria(&criteria, (yyvsp[(1) - (1)].path), NULL, IDMEF_CRITERION_OPERATOR_NOT|IDMEF_CRITERION_OPERATOR_NULL);
+                if ( real_ret < 0 )
+                        YYABORT;
+
+                (yyval.criteria) = criteria;
+        }
     break;
 
   case 11:
-#line 236 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR|IDMEF_CRITERION_OPERATOR_NOCASE; }
+#line 209 "idmef-criteria-string.yac.y"
+    {
+                idmef_criteria_t *criteria;
+
+                real_ret = create_criteria(&criteria, (yyvsp[(2) - (2)].path), NULL, IDMEF_CRITERION_OPERATOR_NULL);
+                if ( real_ret < 0 )
+                        YYABORT;
+
+                (yyval.criteria) = criteria;
+        }
     break;
 
   case 12:
-#line 237 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR|IDMEF_CRITERION_OPERATOR_NOT; }
+#line 222 "idmef-criteria-string.yac.y"
+    {
+                real_ret = idmef_path_new_fast(&cur_path, (yyvsp[(1) - (1)].str));
+                free((yyvsp[(1) - (1)].str));
+
+                if ( real_ret < 0 )
+                        YYABORT;
+
+                path_count = 0;
+                (yyval.path) = cur_path;
+        }
     break;
 
   case 13:
-#line 238 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR|IDMEF_CRITERION_OPERATOR_NOT|IDMEF_CRITERION_OPERATOR_NOCASE; }
+#line 236 "idmef-criteria-string.yac.y"
+    {
+                idmef_criteria_t *criteria;
+                idmef_criterion_value_t *value = NULL;
+
+                real_ret = idmef_criterion_value_new_from_string(&value, cur_path, (yyvsp[(1) - (1)].str), cur_operator);
+                free((yyvsp[(1) - (1)].str));
+
+                if ( real_ret < 0 )
+                        YYABORT;
+
+                real_ret = create_criteria(&criteria, cur_path, value, cur_operator);
+                if ( real_ret < 0 )
+                        YYABORT;
+
+                (yyval.criteria) = criteria;
+        }
     break;
 
   case 14:
-#line 239 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX; }
+#line 257 "idmef-criteria-string.yac.y"
+    {
+                if ( (yyvsp[(2) - (3)].operator) == operator_or )
+                        idmef_criteria_or_criteria((yyvsp[(1) - (3)].criteria), (yyvsp[(3) - (3)].criteria));
+                else
+                        idmef_criteria_and_criteria((yyvsp[(1) - (3)].criteria), (yyvsp[(3) - (3)].criteria));
+
+                (yyval.criteria) = (yyvsp[(1) - (3)].criteria);
+        }
     break;
 
   case 15:
-#line 240 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX|IDMEF_CRITERION_OPERATOR_NOCASE; }
+#line 266 "idmef-criteria-string.yac.y"
+    {
+                (yyval.criteria) = (yyvsp[(2) - (3)].criteria);
+        }
     break;
 
   case 16:
-#line 241 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX|IDMEF_CRITERION_OPERATOR_NOT; }
+#line 270 "idmef-criteria-string.yac.y"
+    {
+                (yyval.criteria) = (yyvsp[(1) - (1)].criteria);
+        }
     break;
 
   case 17:
-#line 242 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX|IDMEF_CRITERION_OPERATOR_NOT|IDMEF_CRITERION_OPERATOR_NOCASE; }
+#line 277 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR; }
     break;
 
   case 18:
-#line 243 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_GREATER; }
+#line 278 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR|IDMEF_CRITERION_OPERATOR_NOCASE; }
     break;
 
   case 19:
-#line 244 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_GREATER|IDMEF_CRITERION_OPERATOR_EQUAL; }
+#line 279 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR|IDMEF_CRITERION_OPERATOR_NOT; }
     break;
 
   case 20:
-#line 245 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_LESSER; }
+#line 280 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR|IDMEF_CRITERION_OPERATOR_NOT|IDMEF_CRITERION_OPERATOR_NOCASE; }
     break;
 
   case 21:
-#line 246 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_LESSER|IDMEF_CRITERION_OPERATOR_EQUAL; }
+#line 281 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX; }
     break;
 
   case 22:
-#line 247 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL; }
+#line 282 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX|IDMEF_CRITERION_OPERATOR_NOCASE; }
     break;
 
   case 23:
-#line 248 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOCASE; }
+#line 283 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX|IDMEF_CRITERION_OPERATOR_NOT; }
     break;
 
   case 24:
-#line 249 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOT; }
+#line 284 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX|IDMEF_CRITERION_OPERATOR_NOT|IDMEF_CRITERION_OPERATOR_NOCASE; }
     break;
 
   case 25:
-#line 250 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOCASE|IDMEF_CRITERION_OPERATOR_NOT; }
+#line 285 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_GREATER; }
     break;
 
   case 26:
-#line 251 "idmef-criteria-string.yac.y"
-    { (yyval.relation) = IDMEF_CRITERION_OPERATOR_NULL; }
+#line 286 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_GREATER|IDMEF_CRITERION_OPERATOR_EQUAL; }
     break;
 
   case 27:
-#line 252 "idmef-criteria-string.yac.y"
+#line 287 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_LESSER; }
+    break;
+
+  case 28:
+#line 288 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_LESSER|IDMEF_CRITERION_OPERATOR_EQUAL; }
+    break;
+
+  case 29:
+#line 289 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL; }
+    break;
+
+  case 30:
+#line 290 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOCASE; }
+    break;
+
+  case 31:
+#line 291 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOT; }
+    break;
+
+  case 32:
+#line 292 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOCASE|IDMEF_CRITERION_OPERATOR_NOT; }
+    break;
+
+  case 33:
+#line 293 "idmef-criteria-string.yac.y"
+    { cur_operator = (yyval.relation) = IDMEF_CRITERION_OPERATOR_NULL; }
+    break;
+
+  case 34:
+#line 294 "idmef-criteria-string.yac.y"
     { real_ret = prelude_error_verbose(PRELUDE_ERROR_IDMEF_CRITERIA_PARSE,
                                                                        "Criteria parser reported: Invalid operator found"); YYERROR; }
     break;
 
-  case 28:
-#line 256 "idmef-criteria-string.yac.y"
+  case 35:
+#line 298 "idmef-criteria-string.yac.y"
     { (yyval.operator) = operator_and; }
     break;
 
-  case 29:
-#line 257 "idmef-criteria-string.yac.y"
+  case 36:
+#line 299 "idmef-criteria-string.yac.y"
     { (yyval.operator) = operator_or; }
     break;
 
 
 /* Line 1267 of yacc.c.  */
-#line 1686 "idmef-criteria-string.yac.c"
+#line 1732 "idmef-criteria-string.yac.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1896,15 +1942,14 @@ yyreturn:
 }
 
 
-#line 260 "idmef-criteria-string.yac.y"
-
+#line 302 "idmef-criteria-string.yac.y"
 
 
 static void yyerror(char *s)  /* Called by yyparse on error */
 {
         real_ret = prelude_error_verbose_make(PRELUDE_ERROR_SOURCE_IDMEF_CRITERIA,
                                               PRELUDE_ERROR_IDMEF_CRITERIA_PARSE,
-                                              "Criteria parser reported: %s", s);
+                                              "IDMEF-Criteria parser: %s", s);
 }
 
 
