@@ -57,7 +57,7 @@
      TOK_RELATION_EQUAL_NOCASE = 273,
      TOK_RELATION_NOT_EQUAL = 274,
      TOK_RELATION_NOT_EQUAL_NOCASE = 275,
-     TOK_RELATION_IS_NULL = 276,
+     TOK_NOT = 276,
      TOK_OPERATOR_AND = 277,
      TOK_OPERATOR_OR = 278,
      TOK_ERROR = 279
@@ -82,7 +82,7 @@
 #define TOK_RELATION_EQUAL_NOCASE 273
 #define TOK_RELATION_NOT_EQUAL 274
 #define TOK_RELATION_NOT_EQUAL_NOCASE 275
-#define TOK_RELATION_IS_NULL 276
+#define TOK_NOT 276
 #define TOK_OPERATOR_AND 277
 #define TOK_OPERATOR_OR 278
 #define TOK_ERROR 279
@@ -92,11 +92,11 @@
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 65 "idmef-criteria-string.yac.y"
+#line 98 "idmef-criteria-string.yac.y"
 {
         char *str;
         int operator;
-        idmef_criterion_t *criterion;
+        idmef_path_t *path;
         idmef_criteria_t *criteria;
         idmef_criterion_operator_t relation;
 }
