@@ -31,6 +31,7 @@
 
 #include "config.h"
 
+#include <arpa/inet.h>
 #include <poll.h>
 #include <stdio.h>
 #include <string.h>
@@ -41,7 +42,6 @@
 #include "ftw_.h"
 #include "getaddrinfo.h"
 #include "getpass.h"
-#include "inet_ntop.h"
 #include "minmax.h"
 #include "pathmax.h"
 #include "regex.h"
