@@ -69,17 +69,19 @@ const char *prelude_connection_pool_get_connection_string(prelude_connection_poo
 
 void prelude_connection_pool_destroy(prelude_connection_pool_t *pool);
 
+prelude_connection_pool_t *prelude_connection_pool_ref(prelude_connection_pool_t *pool);
+
 prelude_connection_pool_flags_t prelude_connection_pool_get_flags(prelude_connection_pool_t *pool);
 
 void prelude_connection_pool_set_flags(prelude_connection_pool_t *pool, prelude_connection_pool_flags_t flags);
 
 void prelude_connection_pool_set_required_permission(prelude_connection_pool_t *pool, prelude_connection_permission_t req_perm);
-         
+
 void prelude_connection_pool_set_data(prelude_connection_pool_t *pool, void *data);
 
 void *prelude_connection_pool_get_data(prelude_connection_pool_t *pool);
 
-int prelude_connection_pool_recv(prelude_connection_pool_t *pool, int timeout, prelude_connection_t **outconn, prelude_msg_t **outmsg);
+int prelude_connection_pool_recv(prelude_connection_pool_t *pool, int timeout, prelude_connection_t **outcon, prelude_msg_t **outmsg);
 
 int prelude_connection_pool_check_event(prelude_connection_pool_t *pool, int timeout,
                                         int (*event_cb)(prelude_connection_pool_t *pool,
@@ -100,5 +102,5 @@ void prelude_connection_pool_set_event_handler(prelude_connection_pool_t *pool,
 #ifdef __cplusplus
  }
 #endif
-         
+
 #endif /* _LIBPRELUDE_PRELUDE_CONNECTION_POOL_H */
