@@ -1,12 +1,12 @@
 /*****
 *
-* Copyright (C) 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by 
+* it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
@@ -52,10 +52,11 @@
 
 
 typedef enum {
-        PRELUDE_LOG_ERR  =  0,
-        PRELUDE_LOG_WARN =  1,
-        PRELUDE_LOG_INFO =  2,
-        PRELUDE_LOG_DEBUG = 3
+        PRELUDE_LOG_CRIT  = -1,
+        PRELUDE_LOG_ERR   =  0,
+        PRELUDE_LOG_WARN  =  1,
+        PRELUDE_LOG_INFO  =  2,
+        PRELUDE_LOG_DEBUG  = 3
 } prelude_log_t;
 
 
@@ -65,37 +66,37 @@ typedef enum {
 } prelude_log_flags_t;
 
 
-         
+
 void _prelude_log_v(prelude_log_t level, const char *file,
-                    const char *function, int line, const char *fmt, va_list ap) 
+                    const char *function, int line, const char *fmt, va_list ap)
                     __attribute__ ((__format__ (__printf__, 5, 0)));
 
 void _prelude_log(prelude_log_t level, const char *file,
-                  const char *function, int line, const char *fmt, ...) 
+                  const char *function, int line, const char *fmt, ...)
                   __attribute__ ((__format__ (__printf__, 5, 6)));
 
-         
+
 #ifdef HAVE_VARIADIC_MACROS
-         
+
 #define prelude_log(level, ...) \
-        _prelude_log(level, __FILE__, __PRELUDE_FUNC__, __LINE__, __VA_ARGS__) 
-        
+        _prelude_log(level, __FILE__, __PRELUDE_FUNC__, __LINE__, __VA_ARGS__)
+
 #define prelude_log_debug(level, ...) \
         _prelude_log(PRELUDE_LOG_DEBUG + level, __FILE__, __PRELUDE_FUNC__, __LINE__, __VA_ARGS__)
 #else
 
-void prelude_log(prelude_log_t level, const char *fmt, ...) 
+void prelude_log(prelude_log_t level, const char *fmt, ...)
                  __attribute__ ((__format__ (__printf__, 2, 3)));
-                 
-void prelude_log_debug(prelude_log_t level, const char *fmt, ...) 
+
+void prelude_log_debug(prelude_log_t level, const char *fmt, ...)
                        __attribute__ ((__format__ (__printf__, 2, 3)));
 
 #endif
 
-         
+
 #define prelude_log_v(level, fmt, ap) \
         _prelude_log_v(level, __FILE__, __PRELUDE_FUNC__, __LINE__, fmt, ap)
-         
+
 #define prelude_log_debug_v(level, fmt, ap) \
         _prelude_log_v(PRELUDE_LOG_DEBUG + level, __FILE__, __PRELUDE_FUNC__, __LINE__, fmt, ap)
 
@@ -116,8 +117,12 @@ void prelude_log_set_callback(void log_cb(prelude_log_t level, const char *str))
 
 int prelude_log_set_logfile(const char *filename);
 
+void _prelude_log_set_abort_level(prelude_log_t level);
+
+int _prelude_log_set_abort_level_from_string(const char *level);
+
 #ifdef __cplusplus
  }
 #endif
-         
+
 #endif /* _LIBPRELUDE_PRELUDE_LOG_H */
