@@ -193,7 +193,7 @@ static ssize_t get_header(prelude_log_t level, char *buf, size_t size)
                 if ( t )
                         len = strftime(buf, size, "%d %b %H:%M:%S ", t);
 
-                ret = snprintf(buf + len, size - len, "(process:%d) %s: ", getpid(), level_to_string(level));
+                ret = snprintf(buf + len, size - len, "(process:%d) %s: ", (int) getpid(), level_to_string(level));
                 if ( ret < 0 || ret >= (size - len) )
                         return -1;
         } else {
@@ -404,7 +404,7 @@ int _prelude_log_set_abort_level_from_string(const char *level)
                 { "DEBUG", PRELUDE_LOG_DEBUG }
         };
 
-        prelude_return_val_if_fail(level != NULL, -1);
+        prelude_return_val_if_fail(level != NULL, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         for ( i = 0; i < sizeof(tbl) / sizeof(*tbl); i++ ) {
                 if ( strncasecmp(tbl[i].prefix, level, strlen(tbl[i].prefix)) == 0 ) {
