@@ -49,41 +49,41 @@ static int config_save_value(config_t *cfg, int rtype, prelude_option_t *last, i
 {
         int ret = 0;
         char buf[1024];
-        
+
         if ( ! (prelude_option_get_type(last) & PRELUDE_OPTION_TYPE_CFG) )
                 return -1;
-        
+
         if ( rtype != PRELUDE_MSG_OPTION_SET && rtype != PRELUDE_MSG_OPTION_DESTROY )
                 return -1;
-        
+
         if ( prelude_option_has_optlist(last) ) {
 
                 if ( prelude_option_get_type(last) & PRELUDE_OPTION_TYPE_CONTEXT )
                         snprintf(buf, sizeof(buf), "%s=%s", option, (value) ? value : "default");
                 else
                         snprintf(buf, sizeof(buf), "%s", option);
-                
+
                 if ( *prev )
                         free(*prev);
-                
+
                 *prev = strdup(buf);
                 if ( ! *prev )
                         return prelude_error_from_errno(errno);
-                
+
                 if ( rtype == PRELUDE_MSG_OPTION_SET )
                         return _config_set(cfg, buf, NULL, NULL, line);
 
                 else if ( is_last_cmd )
                         return _config_del(cfg, buf, NULL);
-                        
+
         }
-        
-        if ( rtype == PRELUDE_MSG_OPTION_SET ) 
+
+        if ( rtype == PRELUDE_MSG_OPTION_SET )
                 ret = _config_set(cfg, *prev, option, value, line);
 
         else if ( is_last_cmd )
                 ret = _config_del(cfg, *prev, option);
-        
+
         return ret;
 }
 
@@ -93,7 +93,7 @@ static int parse_single(void **context, prelude_option_t **last, int is_last_cmd
                         int rtype, const char *option, const char *value, prelude_string_t *out)
 {
         int ret = 0;
-        
+
         *last = prelude_option_search(*last, option, PRELUDE_OPTION_TYPE_WIDE, 0);
         if ( ! *last ) {
                 prelude_string_sprintf(out, "Unknown option: %s.\n", option);
@@ -102,19 +102,19 @@ static int parse_single(void **context, prelude_option_t **last, int is_last_cmd
 
         if ( rtype == PRELUDE_MSG_OPTION_SET )
                 ret = prelude_option_invoke_set(*last, value, out, context);
-        
+
         else if ( is_last_cmd ) {
-		
+
                 if ( rtype == PRELUDE_MSG_OPTION_DESTROY )
                         ret = prelude_option_invoke_destroy(*last, value, out, *context);
-                
+
                 else if ( rtype == PRELUDE_MSG_OPTION_GET )
                         ret = prelude_option_invoke_get(*last, value, out, *context);
-                
+
                 else if ( rtype == PRELUDE_MSG_OPTION_COMMIT )
                         ret = prelude_option_invoke_commit(*last, value, out, *context);
         }
-        
+
         return ret;
 }
 
@@ -128,20 +128,20 @@ static char *option_strsep(char **request)
 
         if ( ! *request )
                 return NULL;
-        
+
         while ( **request ) {
                 if ( ignore == TRUE && **request == ']' )
                         ignore = FALSE;
 
                 if ( ignore == FALSE && **request == '[' )
                         ignore = TRUE;
-                
+
                 if ( ignore == FALSE && **request == '.' ) {
                         **request = 0;
                         *request = *request + 1;
                         return start;
                 }
-                
+
                 (*request)++;
         }
 
@@ -164,16 +164,16 @@ static int parse_request(prelude_client_t *client, int rtype, char *request, pre
         int ret = 0, last_cmd = 0, ent;
         char *str, *value, *prev = NULL, *ptr = NULL;
         unsigned int line = 0;
-        
+
         ret = _config_open(&cfg, prelude_client_get_config_filename(client));
         if ( ret < 0 )
                 return ret;
-        
+
         value = request;
         strsep(&value, "=");
-        
+
         while ( (str = (option_strsep(&request))) ) {
-                
+
                 if ( ! request ) {
                         last_cmd = 1;
                         ptr = value;
@@ -187,17 +187,17 @@ static int parse_request(prelude_client_t *client, int rtype, char *request, pre
                         prelude_string_sprintf(out, "Error parsing option path");
                         break;
                 }
-                
+
                 ret = parse_single(&context, &last, last_cmd, rtype, pname, (ent == 2) ? iname : ptr, out);
                 if ( ret < 0 )
                         break;
-                
+
                 config_save_value(cfg, rtype, last, last_cmd, &prev, pname, (ent == 2) ? iname : ptr, &line);
         }
-        
+
         _config_close(cfg);
         free(prev);
-        
+
         return ret;
 }
 
@@ -206,11 +206,11 @@ static int parse_request(prelude_client_t *client, int rtype, char *request, pre
 static void send_string(prelude_msgbuf_t *msgbuf, prelude_string_t *out, int type)
 {
         size_t len;
-        
+
         len = prelude_string_is_empty(out) ? 0 : (prelude_string_get_len(out) + 1);
         if ( type == PRELUDE_MSG_OPTION_VALUE && ! len )
                 return;
-        
+
         prelude_msgbuf_set(msgbuf, type, len, prelude_string_get_string(out));
 }
 
@@ -221,13 +221,13 @@ static void send_error(prelude_msgbuf_t *msgbuf, const char *fmt, ...)
         int ret;
         va_list ap;
         prelude_string_t *out;
-        
+
         va_start(ap, fmt);
 
         ret = prelude_string_new(&out);
         if ( ret < 0 )
                 return;
-        
+
         prelude_string_vprintf(out, fmt, ap);
 
         va_end(ap);
@@ -246,11 +246,11 @@ static int read_option_request(prelude_client_t *client, prelude_msgbuf_t *msgbu
         uint32_t len, hop;
         int ret, type = -1;
         prelude_string_t *out;
-        
+
         while ( prelude_msg_get(msg, &tag, &len, &buf) == 0 ) {
-                
+
                 switch (tag) {
-                        
+
                 case PRELUDE_MSG_OPTION_SET:
                 case PRELUDE_MSG_OPTION_GET:
                 case PRELUDE_MSG_OPTION_COMMIT:
@@ -262,81 +262,81 @@ static int read_option_request(prelude_client_t *client, prelude_msgbuf_t *msgbu
                         ret = prelude_extract_uint32_safe(&hop, buf, len);
                         if ( ret < 0 )
                                 return ret;
-                        
+
                         hop = htonl(hop - 1);
                         prelude_msgbuf_set(msgbuf, tag, len, &hop);
                         break;
-                        
-                case PRELUDE_MSG_OPTION_TARGET_ID:                        
+
+                case PRELUDE_MSG_OPTION_TARGET_ID:
                 case PRELUDE_MSG_OPTION_TARGET_INSTANCE_ID:
-                case PRELUDE_MSG_OPTION_REQUEST_ID:                        
+                case PRELUDE_MSG_OPTION_REQUEST_ID:
                         prelude_msgbuf_set(msgbuf, tag, len, buf);
                         break;
-                        
+
                 case PRELUDE_MSG_OPTION_LIST:
                         return prelude_option_wide_send_msg(msgbuf, client);
-                        
+
                 case PRELUDE_MSG_OPTION_VALUE:
                         ret = prelude_extract_characters_safe((const char **) &request, buf, len);
                         if ( ret < 0 )
                                 return ret;
-                        
+
                         if ( type < 0 || ! request ) {
                                 send_error(msgbuf, "No request specified");
                                 return -1;
                         }
-                        
+
                         ret = prelude_string_new(&out);
                         if ( ret < 0 )
                                 return ret;
-                        
+
                         ret = parse_request(client, type, request, out);
                         send_string(msgbuf, out, (ret < 0) ? PRELUDE_MSG_OPTION_ERROR : PRELUDE_MSG_OPTION_VALUE);
 
                         prelude_string_destroy(out);
                         break;
-                        
+
                 default:
                         send_error(msgbuf, "Unknown option tag: %d", tag);
                         return -1;
                 }
         }
-        
+
         return 0;
 }
 
 
 
-static int read_option_list(prelude_msg_t *msg, prelude_option_t *opt, uint64_t *source_id) 
+static int read_option_list(prelude_msg_t *msg, prelude_option_t *opt, uint64_t *source_id)
 {
         int ret;
         void *buf;
         uint8_t tag;
-        const char *tmp;
-        uint32_t dlen, tmpint;
+        const char *tmp = NULL;
+        uint32_t dlen, tmpint = 0;
         prelude_option_t *newopt;
-        
+
         if ( ! opt )
                 return -1;
-        
+
         while ( (ret = prelude_msg_get(msg, &tag, &dlen, &buf)) == 0 ) {
-                
+
                 switch (tag) {
-                
+
                 case PRELUDE_MSG_OPTION_START:
                         ret = prelude_option_new(opt, &newopt);
                         if ( ret < 0 )
                                 break;
-                        
+
                         read_option_list(msg, newopt, source_id);
                         break;
-                        
+
                 case PRELUDE_MSG_OPTION_END:
                         return 0;
-                        
+
                 case PRELUDE_MSG_OPTION_VALUE:
                         ret = prelude_extract_characters_safe(&tmp, buf, dlen);
-                        if ( ret < 0 ) 
+                        if ( ret < 0 )
                                 return ret;
 
                         prelude_option_set_value(opt, tmp);
@@ -349,7 +349,7 @@ static int read_option_list(prelude_msg_t *msg, prelude_option_t *opt, uint64_t
 
                         prelude_option_set_longopt(opt, tmp);
                         break;
-                        
+
                 case PRELUDE_MSG_OPTION_DESC:
                         ret = prelude_extract_characters_safe(&tmp, buf, dlen);
                         if ( ret < 0 )
@@ -357,20 +357,20 @@ static int read_option_list(prelude_msg_t *msg, prelude_option_t *opt, uint64_t
 
                         prelude_option_set_description(opt, tmp);
                         break;
-                        
+
                 case PRELUDE_MSG_OPTION_HELP:
                         ret = prelude_extract_characters_safe(&tmp, buf, dlen);
                         if ( ret < 0 )
                                 return ret;
-                        
+
                         prelude_option_set_help(opt, tmp);
                         break;
-                        
+
                 case PRELUDE_MSG_OPTION_INPUT_VALIDATION:
                         ret = prelude_extract_characters_safe(&tmp, buf, dlen);
                         if ( ret < 0 )
                                 return ret;
-                        
+
                         prelude_option_set_input_validation_regex(opt, tmp);
                         break;
 
@@ -389,7 +389,7 @@ static int read_option_list(prelude_msg_t *msg, prelude_option_t *opt, uint64_t
 
                         prelude_option_set_type(opt, tmpint);
                         break;
-                        
+
                 case PRELUDE_MSG_OPTION_INPUT_TYPE:
                         ret = prelude_extract_uint32_safe(&tmpint, buf, dlen);
                         if ( ret < 0 )
@@ -397,7 +397,7 @@ static int read_option_list(prelude_msg_t *msg, prelude_option_t *opt, uint64_t
 
                         prelude_option_set_input_type(opt, tmpint);
                         break;
-                        
+
                 default:
                         /*
                          * for compatibility purpose, don't return an error on unknow tag.
@@ -414,9 +414,9 @@ static int read_option_list(prelude_msg_t *msg, prelude_option_t *opt, uint64_t
 int prelude_option_process_request(prelude_client_t *client, prelude_msg_t *msg, prelude_msgbuf_t *out)
 {
         uint8_t tag;
-        
+
         tag = prelude_msg_get_tag(msg);
-        
+
         if ( tag != PRELUDE_MSG_OPTION_REQUEST )
                 return -1;
 
@@ -429,12 +429,12 @@ int prelude_option_process_request(prelude_client_t *client, prelude_msg_t *msg,
 
 
 int prelude_option_push_request(prelude_msgbuf_t *msg, int type, const char *request)
-{        
+{
         prelude_msgbuf_set(msg, type, 0, 0);
-        
-        if ( request ) 
+
+        if ( request )
                 prelude_msgbuf_set(msg, PRELUDE_MSG_OPTION_VALUE, strlen(request) + 1, request);
-                
+
         return 0;
 }
 
@@ -445,7 +445,7 @@ int prelude_option_new_request(prelude_msgbuf_t *msgbuf,
 {
         int i;
         uint32_t hop, instance_id = 0;
-        
+
         prelude_msg_set_tag(prelude_msgbuf_get_msg(msgbuf), PRELUDE_MSG_OPTION_REQUEST);
 
         /*
@@ -456,13 +456,13 @@ int prelude_option_new_request(prelude_msgbuf_t *msgbuf,
 
         for ( i = 0; i < size; i++ )
                 target_id[i] = prelude_hton64(target_id[i]);
-        
+
         request_id = htonl(request_id);
         prelude_msgbuf_set(msgbuf, PRELUDE_MSG_OPTION_REQUEST_ID, sizeof(request_id), &request_id);
         prelude_msgbuf_set(msgbuf, PRELUDE_MSG_OPTION_TARGET_ID, i * sizeof(*target_id), target_id);
         prelude_msgbuf_set(msgbuf, PRELUDE_MSG_OPTION_TARGET_INSTANCE_ID, sizeof(instance_id), &instance_id);
         prelude_msgbuf_set(msgbuf, PRELUDE_MSG_OPTION_HOP, sizeof(hop), &hop);
-        
+
         return 0;
 }
 
@@ -475,7 +475,7 @@ int prelude_option_recv_reply(prelude_msg_t *msg, uint64_t *source_id,
         uint8_t tag;
         uint32_t dlen;
         int ret, type = -1;
-        
+
         *value = NULL;
 
         while ( (ret = prelude_msg_get(msg, &tag, &dlen, &buf)) == 0 ) {
@@ -483,20 +483,20 @@ int prelude_option_recv_reply(prelude_msg_t *msg, uint64_t *source_id,
                 switch (tag) {
                 case PRELUDE_MSG_OPTION_HOP:
                         break;
-                        
+
                 case PRELUDE_MSG_OPTION_REQUEST_ID:
                         type = PRELUDE_OPTION_REPLY_TYPE_SET;
-                        
+
                         ret = prelude_extract_uint32_safe(request_id, buf, dlen);
                         if ( ret < 0 )
                                 return ret;
-                        
+
                         break;
-                        
+
                 case PRELUDE_MSG_OPTION_VALUE:
                         type = PRELUDE_OPTION_REPLY_TYPE_GET;
-                        
-                        ret = prelude_extract_characters_safe((const char **) value, buf, dlen);                        
+
+                        ret = prelude_extract_characters_safe((const char **) value, buf, dlen);
                         if ( ret < 0 )
                                 return ret;
                         break;
@@ -504,15 +504,15 @@ int prelude_option_recv_reply(prelude_msg_t *msg, uint64_t *source_id,
                 case PRELUDE_MSG_OPTION_ERROR:
                         type = PRELUDE_OPTION_REPLY_TYPE_ERROR;
                         if ( ! dlen ) {
-                                *value = "No error message";	
+                                *value = "No error message";
                                 break;
                         }
-			
+
                         ret = prelude_extract_characters_safe((const char **) value, buf, dlen);
                         if ( ret < 0 )
                                 return ret;
                         break;
-                                                
+
                 case PRELUDE_MSG_OPTION_TARGET_ID:
                         if ( dlen % sizeof(uint64_t) != 0 || dlen < (2 * sizeof(uint64_t)) )
                                 return -1;
@@ -520,22 +520,22 @@ int prelude_option_recv_reply(prelude_msg_t *msg, uint64_t *source_id,
                         *source_id = prelude_extract_uint64((unsigned char *) buf + (dlen - sizeof(uint64_t)));
                         break;
 
-                case PRELUDE_MSG_OPTION_LIST:                        
+                case PRELUDE_MSG_OPTION_LIST:
                         type = PRELUDE_OPTION_REPLY_TYPE_LIST;
 
                         ret = prelude_option_new(NULL, (prelude_option_t **) value);
                         if ( ret < 0 )
                                 return ret;
-                        
+
                         ret = read_option_list(msg, *value, NULL);
                         if ( ret < 0 )
                                 return ret;
                         break;
-                        
+
                 default:
                         prelude_log(PRELUDE_LOG_WARN, "unknown option tag %d.\n", tag);
                 }
         }
-        
+
         return type;
 }
